import {
  IconButton,
  Box,
  Flex,
  Icon,
  useColorModeValue,
  Text,
  Drawer,
  DrawerContent,
  useDisclosure,
  BoxProps,
  FlexProps,
  Container,
  Button,
  VStack,
} from "@chakra-ui/react";
import { FiHome, FiTrendingUp, FiMenu } from "react-icons/fi";
import { IconType } from "react-icons";
import Link from "next/link";
import { useRouter } from "next/router";
import ProfileMenu from "@/components/ProfileMenu";
import { ReactNode } from "react";
import useUser from "@/hooks/useUser";

interface LinkItemProps {
  name: string;
  icon: IconType;
  href: string;
  role?: string[];
}

const Topbar = () => {
  return (
    <Box
      bgColor="white"
      width="100%"
      borderBottom="1px"
      alignItems="center"
      height={16}
      px={4}
      display="flex"
      justifyContent="flex-end"
      borderBottomColor={useColorModeValue("gray.200", "gray.700")}
    >
      <ProfileMenu />
    </Box>
  );
};

const LinkItems: Array<LinkItemProps> = [
  {
    name: "Projekty",
    icon: FiHome,
    href: "/admin/projects",
    role: ["admin", "editor", "reader"],
  },
  {
    name: "Uživatelé",
    icon: FiTrendingUp,
    href: "/admin/users",
    role: ["admin"],
  },
];

const Layout = ({ children }: { children: ReactNode }) => {
  const { isOpen, onOpen, onClose } = useDisclosure();
  return (
    <Box
      minH="100vh"
      display={{ base: "block", md: "flex" }}
      bg={useColorModeValue("gray.100", "gray.900")}
    >
      <SidebarContent
        onClose={() => onClose}
        display={{ base: "none", md: "block" }}
      />
      <Drawer
        isOpen={isOpen}
        placement="left"
        onClose={onClose}
        returnFocusOnClose={false}
        onOverlayClick={onClose}
        size="full"
      >
        <DrawerContent>
          <SidebarContent onClose={onClose} />
        </DrawerContent>
      </Drawer>
      <MobileNav display={{ base: "flex", md: "none" }} onOpen={onOpen} />
      <Box display="flex" flexDirection="column" width="100%">
        <Topbar></Topbar>
        <Container maxW="full" paddingY={4}>
          {children}
        </Container>
      </Box>
    </Box>
  );
};

export default Layout;

interface SidebarProps extends BoxProps {
  onClose: () => void;
}

const SidebarContent = ({ onClose, ...rest }: SidebarProps) => {
  const router = useRouter();

  const { role } = useUser();

  return (
    <Box
      bg={useColorModeValue("white", "gray.900")}
      borderRight="1px"
      px={8}
      borderRightColor={useColorModeValue("gray.200", "gray.700")}
      post={{ base: "fixed", md: "static" }}
      {...rest}
    >
      <Box height={16}></Box>
      {/* <CloseButton display={{ base: "flex", md: "none" }} onClick={onClose} /> */}
      <VStack direction="column" alignItems="stretch" spacing={4}>
        {LinkItems.map((link) => {
          if (!link.role?.includes(role)) return;
          return (
            <Link href={link.href} key={link.href}>
              <Button
                key={link.name}
                px={16}
                width="full"
                variant="outline"
                colorScheme={router.pathname === link.href ? "blue" : "gray"}
                leftIcon={<link.icon />}
              >
                {link.name}
              </Button>
            </Link>
          );
        })}
      </VStack>
    </Box>
  );
};

interface NavItemProps extends FlexProps {
  icon: IconType;
  children: any;
  href: string;
}
const NavItem = ({ icon, href, children, ...rest }: NavItemProps) => {
  const router = useRouter();
  const isActive = router.pathname === href;

  return (
    <Link href={href}>
      <Box style={{ textDecoration: "none" }} _focus={{ boxShadow: "none" }}>
        <Flex
          align="center"
          py="4"
          px="8"
          borderRadius="lg"
          role="group"
          cursor="pointer"
          _hover={{
            bg: "cyan.400",
            color: "white",
          }}
          bg={isActive ? "cyan.400" : "transparent"}
          {...rest}
        >
          {icon && (
            <Icon
              mr="4"
              fontSize="16"
              _groupHover={{
                color: "white",
              }}
              as={icon}
            />
          )}
          {children}
        </Flex>
      </Box>
    </Link>
  );
};

interface MobileProps extends FlexProps {
  onOpen: () => void;
}
const MobileNav = ({ onOpen, ...rest }: MobileProps) => {
  return (
    <Flex
      ml={{ base: 0, md: 60 }}
      px={{ base: 4, md: 24 }}
      height="20"
      alignItems="center"
      bg={useColorModeValue("white", "gray.900")}
      borderBottomWidth="1px"
      borderBottomColor={useColorModeValue("gray.200", "gray.700")}
      justifyContent="flex-start"
      {...rest}
    >
      <IconButton
        variant="outline"
        onClick={onOpen}
        aria-label="open menu"
        icon={<FiMenu />}
      />

      <Text fontSize="2xl" ml="8" fontFamily="monospace" fontWeight="bold">
        Logo
      </Text>
    </Flex>
  );
};
