import { useEffect, useRef, useState } from "react";
import {
  Button,
  Heading,
  Tab,
  TabList,
  Card,
  CardBody,
  TabPanels,
  TabPanel,
  FormControl,
  FormLabel,
  Input,
  Tabs,
  IconButton,
  CardHeader,
  Box,
  Textarea,
} from "@chakra-ui/react";
import { FiPlusCircle } from "react-icons/fi";
import { SaveType } from "@/types";
import Scene from "@/components/Scene";
import { v4 as uuidv4 } from "uuid";
import { Control, UseFormRegister, useFieldArray } from "react-hook-form";
import SceneTab from "@/components/SceneTab";
import { useRouter } from "next/router";
import { Project } from "@/types";
import useUser from "@/hooks/useUser";

type Props = {
  register: UseFormRegister<Project>;
  control: Control<Project, any>;
  onSubmit: any;
  isLoading: boolean;
};

const ProjectForm = ({ register, onSubmit, control, isLoading }: Props) => {
  const router = useRouter();

  const [tabIndex, setTabIndex] = useState(0);
  const [sceneIndex, setSceneIndex] = useState(0);
  const [isFirstRender, setIsFirstRender] = useState(true);

  const refSubmitButton = useRef<HTMLButtonElement>(null);

  const { fields, append, remove } = useFieldArray({
    control,
    name: "scenes",
  });

  useEffect(() => {
    const { tabIndex, sceneIndex } = router.query;
    if (tabIndex) setTabIndex(Number(tabIndex));
    if (sceneIndex) setSceneIndex(Number(sceneIndex));
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  useEffect(() => {
    if (!isFirstRender) {
      router.query.tabIndex = tabIndex.toString();
      router.query.sceneIndex = sceneIndex.toString();
      router.push(router);
    }
    setIsFirstRender(false);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [tabIndex, sceneIndex]);

  const { isReader } = useUser();

  return (
    <Box as="form" onSubmit={onSubmit}>
      <Tabs
        variant="soft-rounded"
        colorScheme="blue"
        index={tabIndex}
        onChange={(index) => setTabIndex(index)}
      >
        <TabList>
          <Tab>Základní informace</Tab>
          <Tab>Scény</Tab>
        </TabList>
        <TabPanels>
          <TabPanel>
            <Card>
              <CardHeader>
                <Heading size="md" marginBottom={4}>
                  Základní informace
                </Heading>
              </CardHeader>
              <CardBody>
                <FormControl mb={4}>
                  <FormLabel htmlFor="firstName">Název projektu</FormLabel>
                  <Input
                    id="name"
                    type="text"
                    {...register("name")}
                    readOnly={isReader}
                  />
                </FormControl>
                <FormControl>
                  <FormLabel htmlFor="firstName">Popisek projektu</FormLabel>
                  <Textarea
                    id="description"
                    {...register("description")}
                    readOnly={isReader}
                  />
                </FormControl>
              </CardBody>
            </Card>
          </TabPanel>
          <TabPanel>
            <Tabs
              variant="soft-rounded"
              colorScheme="blue"
              index={sceneIndex}
              onChange={(index) => setSceneIndex(index)}
            >
              <TabList>
                {fields.map((field, index) => (
                  <SceneTab control={control} index={index} key={field.id} />
                ))}
                {!isReader && (
                  <IconButton
                    aria-label="Přidat scénu"
                    icon={<FiPlusCircle />}
                    onClick={() =>
                      append({
                        name: "Nová scéna",
                        description: '',
                        id: uuidv4(),
                        cols: 2,
                        saveType: SaveType.DEFAULT,
                      })
                    }
                  />
                )}
              </TabList>
              <TabPanels>
                {fields.map((field, index) => (
                  <TabPanel key={field.id}>
                    <Scene
                      remove={remove}
                      index={index}
                      register={register}
                      control={control}
                    />
                  </TabPanel>
                ))}
              </TabPanels>
            </Tabs>
          </TabPanel>
        </TabPanels>
      </Tabs>
      <Button
        type="submit"
        isLoading={isLoading}
        colorScheme="blue"
        ref={refSubmitButton}
      >
        Uložit
      </Button>
    </Box>
  );
};

export default ProjectForm;
