import {
  AlertDialog,
  AlertDialogContent,
  AlertDialogFooter,
  AlertDialogHeader,
  AlertDialogOverlay,
  Button,
  Card,
  CardBody,
  Flex,
  FormControl,
  FormHelperText,
  FormLabel,
  Grid,
  GridItem,
  Heading,
  IconButton,
  Input,
  Stack,
  useClipboard,
  useDisclosure,
  AlertDialogBody,
  Modal,
  ModalOverlay,
  ModalContent,
  ModalCloseButton,
  ModalBody,
  ModalFooter,
  ModalHeader,
  NumberInput,
  NumberInputStepper,
  NumberIncrementStepper,
  NumberDecrementStepper,
  NumberInputField,
  HStack,
  RadioGroup,
  Radio,
  Tooltip,
  Box,
  Textarea,
} from "@chakra-ui/react";
import { Fragment, useEffect, useRef } from "react";
import {
  Control,
  Controller,
  UseFormRegister,
  useFieldArray,
  useWatch,
} from "react-hook-form";
import { v4 as uuidv4 } from "uuid";
import SceneItem from "../SceneItem";
import { Project } from "@/types";
import { FaCheck, FaCog, FaCopy, FaPlus, FaTrash } from "react-icons/fa";
import { useRouter } from "next/router";
import useUser from "@/hooks/useUser";

type SceneProps = {
  index: number;
  register: UseFormRegister<Project>;
  control: Control<Project, any>;
  remove: (index: number) => void;
};

const Scene = ({ index, register, control, remove }: SceneProps) => {
  const { isOpen, onOpen, onClose } = useDisclosure();
  const {
    isOpen: isSettingsOpen,
    onOpen: onSettingsOpen,
    onClose: onSettingsClose,
  } = useDisclosure();
  const cancelRef = useRef<any>();

  const parent = `scenes.${index}.layout`;
  const {
    fields,
    append,
    remove: removeChild,
  } = useFieldArray({
    control,
    name: parent as any,
  });

  const router = useRouter();

  const projectId = router.query.id;
  console.log(projectId);

  const { id: sceneId, cols } = useWatch({
    control,
    name: `scenes.${index}`,
  });

  const deleteScene = () => {
    remove(index);
    onClose();
  };

  const { onCopy, setValue, hasCopied } = useClipboard("");

  const projectUrl = process.env.NEXT_PUBLIC_PROJECT_URL || "localhost:3000";

  useEffect(() => {
    setValue(`${projectUrl}/api/projects/${projectId}/${sceneId}/data`);
  }, [projectId, sceneId, setValue, projectUrl]);

  const { isReader } = useUser();

  const isMissingProperties = !projectUrl || !projectId || !sceneId;

  return (
    <Fragment>
      <Heading mb={2} size="md">
        Nastavení scény
      </Heading>
      <Card mb={4}>
        <CardBody>
          <FormControl mb={4}>
            <FormLabel>Název scény</FormLabel>
            <Input {...register(`scenes.${index}.name`)} readOnly={isReader} />
            <FormHelperText>Uveď název scény</FormHelperText>
          </FormControl>
          <FormControl mb={4}>
            <FormLabel htmlFor="firstName">Popisek scény</FormLabel>
            <Textarea
              id="description"
              {...register(`scenes.${index}.description`)}
              readOnly={isReader}
            />
          </FormControl>
          {/* <FormControl mb={8}>
            <FormLabel>Typ scény</FormLabel>
            <Controller
              name={`scenes.${index}.saveType`}
              control={control}
              render={({ field: { ref, ...restField } }) => (
                <RadioGroup {...restField}>
                  <Stack direction="row">
                    <Radio value={SaveType.DEFAULT}>Výchozí</Radio>
                    <Radio value={SaveType.ITERATED}>Iterovaná</Radio>
                  </Stack>
                </RadioGroup>
              )}
            />
            <FormHelperText>
              Vyber, jaký typ scény potřebuješ. Výchozí typ scény udržuje
              výsledek pro vMix ve formátu jednoho objektu. Iterovaný typ scény
              udržuje výsledek pro vMix ve formátu mnoha objektů v poli.
            </FormHelperText>
          </FormControl> */}
          <Flex gap={2}>
            <Tooltip
              label={
                isMissingProperties
                  ? "Nelze zkopírovat URL. Chybí identifikátory projektu a scény. Opravdu jsi projekt a scénu uložil?"
                  : "Tlačítko zkopíruje URL adresu, kterou je poté možné vložit do programu vMix."
              }
            >
              <IconButton
                colorScheme="blue"
                aria-label="Kopírovat URL"
                isDisabled={isMissingProperties}
                onClick={onCopy}
                icon={hasCopied ? <FaCheck /> : <FaCopy />}
              />
            </Tooltip>
            {!isReader && (
              <IconButton
                colorScheme="red"
                aria-label="Smazat scénu"
                onClick={onOpen}
                icon={<FaTrash />}
              />
            )}
          </Flex>
        </CardBody>
      </Card>
      <Stack alignItems="flex-start" width="100%">
        {!isReader && (
          <HStack sx={{ marginBottom: 4 }}>
            <IconButton
              colorScheme="blue"
              onClick={() => append({ name: "Nový box", id: uuidv4() })}
              icon={<FaPlus />}
              aria-label="Přidat objekt"
            />
            <IconButton
              colorScheme="blue"
              onClick={onSettingsOpen}
              icon={<FaCog />}
              aria-label="Otevřít dialog s nastavením"
            />
          </HStack>
        )}
        <Grid templateColumns={`repeat(${cols}, 1fr)`} gap={6} w="100%">
          {fields.map((field, index) => (
            <GridItem key={field.id}>
              <SceneItem
                index={index}
                register={register}
                control={control}
                parent={parent}
                remove={removeChild}
              />
            </GridItem>
          ))}
        </Grid>
      </Stack>
      <AlertDialog
        isOpen={isOpen}
        onClose={onClose}
        leastDestructiveRef={cancelRef}
      >
        <AlertDialogOverlay>
          <AlertDialogContent>
            <AlertDialogHeader fontSize="lg" fontWeight="bold">
              Smazat scénu
            </AlertDialogHeader>

            <AlertDialogBody>Chceš smazat scénu?</AlertDialogBody>

            <AlertDialogFooter>
              <Button ref={cancelRef} onClick={onClose}>
                Zrušit
              </Button>
              <Button colorScheme="red" onClick={deleteScene} ml={3}>
                Smazat
              </Button>
            </AlertDialogFooter>
          </AlertDialogContent>
        </AlertDialogOverlay>
      </AlertDialog>
      <Modal isOpen={isSettingsOpen} onClose={onSettingsClose}>
        <ModalOverlay />
        <ModalContent>
          <ModalHeader>Nastavení vzhledu</ModalHeader>
          <ModalCloseButton />
          <ModalBody>
            <FormControl mb={8}>
              <FormLabel>Počet sloupců</FormLabel>
              <Controller
                name={`scenes.${index}.cols`}
                control={control}
                render={({ field: { ref, ...restField } }) => (
                  <NumberInput {...restField} min={1} max={12}>
                    <NumberInputField ref={ref} name={restField.name} />
                    <NumberInputStepper>
                      <NumberIncrementStepper />
                      <NumberDecrementStepper />
                    </NumberInputStepper>
                  </NumberInput>
                )}
              />
              <FormHelperText>Uveď počet sloupců</FormHelperText>
            </FormControl>
          </ModalBody>
          <ModalFooter>
            <Button colorScheme="blue" mr={3} onClick={onSettingsClose}>
              Zavřít
            </Button>
          </ModalFooter>
        </ModalContent>
      </Modal>
    </Fragment>
  );
};

export default Scene;
