import {
  Button,
  Card,
  CardBody,
  CardFooter,
  IconButton,
  Modal,
  ModalBody,
  ModalCloseButton,
  ModalContent,
  ModalFooter,
  ModalHeader,
  ModalOverlay,
  Stack,
  useDisclosure,
  Box,
} from "@chakra-ui/react";
import { Fragment } from "react";
import {
  Control,
  Controller,
  UseFormRegister,
  useFieldArray,
  useWatch,
} from "react-hook-form";
import { v4 as uuidv4 } from "uuid";
import SceneField from "../SceneField";
import { Project, FieldType } from "@/types";
import { FaPlus, FaTrash, FaBrush } from "react-icons/fa";
import useUser from "@/hooks/useUser";
import { RgbaStringColorPicker } from "react-colorful";

type SceneItemProps = {
  register: UseFormRegister<Project>;
  parent: string;
  index: number;
  control: Control<Project, any>;
  remove: (index: number) => void;
};

const SceneItem = ({
  index,
  register,
  control,
  parent,
  remove,
}: SceneItemProps) => {
  const {
    fields,
    append,
    remove: removeChild,
  } = useFieldArray({
    control,
    name: `${parent}.${index}.fields` as any,
  });

  const { isOpen, onOpen, onClose } = useDisclosure();

  const color = useWatch({
    control,
    name: `${parent}.${index}.color` as any,
  });

  const {
    isOpen: isItemModalOpen,
    onOpen: onItemModalOpen,
    onClose: onItemModalClose,
  } = useDisclosure();

  const handleAppend = (type: FieldType) => {
    append({ name: "Nový box", id: uuidv4(), type: type });
    onClose();
  };

  const { isReader } = useUser();

  return (
    <Fragment>
      <Card
        sx={{
          position: "relative",
          borderColor: color,
          borderWidth: 4,
          borderStyle: "dashed",
        }}
      >
        <Box sx={{ position: "absolute", top: 2, right: 2, zIndex: 1 }}>
          <IconButton
            icon={<FaBrush />}
            onClick={onItemModalOpen}
            colorScheme="blue"
            size="xs"
            aria-label="Otevřít menu s úpravami"
          />
        </Box>
        <CardBody>
          <Stack>
            {fields.map((field, i) => (
              <SceneField
                key={field.id}
                register={register}
                parent={`${parent}.${index}.fields`}
                remove={removeChild}
                index={i}
                control={control}
              />
            ))}
          </Stack>
        </CardBody>
        <CardFooter gap={2}>
          {!isReader && (
            <Fragment>
              <IconButton
                colorScheme="blue"
                onClick={onOpen}
                icon={<FaPlus />}
                aria-label="Přidat objekt"
              ></IconButton>
              <IconButton
                colorScheme="red"
                aria-label="Smazat scénu"
                onClick={() => remove(index)}
                icon={<FaTrash />}
              ></IconButton>
            </Fragment>
          )}
        </CardFooter>
      </Card>
      <Modal isOpen={isOpen} onClose={onClose}>
        <ModalOverlay />
        <ModalContent>
          <ModalHeader>Přidání pole</ModalHeader>
          <ModalCloseButton />
          <ModalBody>
            <Stack>
              <Button onClick={() => handleAppend(FieldType.TEXT)}>
                Přidat textové pole
              </Button>
              <Button onClick={() => handleAppend(FieldType.SELECT)}>
                Přidat pole s výběrem
              </Button>
              <Button onClick={() => handleAppend(FieldType.NUMBER)}>
                Přidat číselné pole
              </Button>
            </Stack>
          </ModalBody>
          <ModalFooter>
            <Button colorScheme="blue" mr={3} onClick={onClose}>
              Zavřít
            </Button>
          </ModalFooter>
        </ModalContent>
      </Modal>
      <Modal isOpen={isItemModalOpen} onClose={onItemModalClose}>
        <ModalOverlay />
        <ModalContent>
          <ModalHeader>Úprava boxu</ModalHeader>
          <ModalCloseButton />
          <ModalBody>
            <Controller
              control={control}
              name={`${parent}.${index}.color` as any}
              render={({ field: { onChange, value } }) => (
                <RgbaStringColorPicker color={value} onChange={onChange} />
              )}
            />
          </ModalBody>
          <ModalFooter>
            <Button colorScheme="blue" mr={3} onClick={onItemModalClose}>
              Zavřít
            </Button>
          </ModalFooter>
        </ModalContent>
      </Modal>
    </Fragment>
  );
};

export default SceneItem;
