import {
  Alert,
  AlertIcon,
  Box,
  Button,
  Card,
  CardBody,
  Container,
  FormControl,
  FormLabel,
  HStack,
  Heading,
  Input,
  Stack,
  Step,
  StepIcon,
  StepIndicator,
  StepNumber,
  StepSeparator,
  StepStatus,
  StepTitle,
  Stepper,
  useSteps,
} from "@chakra-ui/react";
import { PasswordField } from "@/components/PasswordField";
import { useForm } from "react-hook-form";
import useSWR from "swr";
import fetcher from "@/utils/fetcher";
import { useRouter } from "next/router";
import { signIn } from "next-auth/react";
import LoadingScreen from "@/components/LoadingScreen";
import { Fragment, useState } from "react";
import { FiArrowLeft } from "react-icons/fi";

type Inputs = {
  email: string;
  password: string;
  repeatPassword: string;
  firstName: string;
  lastName: string;
};

const SignUp = () => {
  const router = useRouter();
  const { data, isLoading } = useSWR("/api/users/init-check", fetcher);
  const [loading, setLoading] = useState(false);

  const { register, handleSubmit } = useForm<Inputs>();

  const onSubmit = async (data: Inputs) => {
    setLoading(true);
    const res = await fetch("/api/auth/sign-up", {
      method: "POST",
      body: JSON.stringify(data),
      headers: { "Content-Type": "application/json" },
    });
    if (res.ok) {
      await signIn("credentials", {
        email: data.email,
        password: data.password,
        callbackUrl: "/admin/users",
      });
    }
    setLoading(false);
  };

  const StepOne = () => (
    <Fragment>
      <FormControl isRequired>
        <FormLabel htmlFor="email">E-mailová adresa</FormLabel>
        <Input
          id="email"
          type="email"
          {...register("email", {
            required: "Pole je vyžadováno",
          })}
        />
      </FormControl>
      <PasswordField
        {...register("password", {
          required: "Pole je vyžadováno",
        })}
      />

      <PasswordField
        {...register("repeatPassword", {
          required: "Pole je vyžadováno",
        })}
      />
      <Button
        onClick={goToNext}
        isLoading={loading}
        colorScheme="blue"
        variant="outline"
      >
        Pokračovat
      </Button>
    </Fragment>
  );

  const StepTwo = () => (
    <Fragment>
      <FormControl>
        <FormLabel htmlFor="firstName">Jméno</FormLabel>
        <Input id="firstName" type="text" {...register("firstName")} />
      </FormControl>
      <FormControl>
        <FormLabel htmlFor="lastName">Přijmení</FormLabel>
        <Input id="lastName" type="text" {...register("lastName")} />
      </FormControl>
      <HStack>
        <Button onClick={goToPrevious} w="30%" leftIcon={<FiArrowLeft />}>
          Zpět
        </Button>
        <Button type="submit" isLoading={loading} colorScheme="blue" w="70%">
          Registrovat se
        </Button>
      </HStack>
    </Fragment>
  );

  const steps = [
    {
      title: "Přihlašovací údaje",
      content: <StepOne />,
    },
    {
      title: "Dodatečné údaje",
      content: <StepTwo />,
    },
  ];

  const { activeStep, goToNext, goToPrevious } = useSteps({
    index: 1,
    count: steps.length,
  });

  if (isLoading) return <LoadingScreen />;

  if (data === true) router.push("/auth/sign-in");

  return (
    <Box bgColor="gray.50" minHeight="100vh">
      <Container
        maxW="xl"
        py={{ base: "12", md: "24" }}
        px={{ base: "0", sm: "8" }}
      >
        <Stack spacing="8">
          <Heading size="md">Prvotní registrace</Heading>
          <Alert status="info">
            <AlertIcon />V aplikaci ještě nejsou žádní uživatelé. Prosím, založ
            si prvního uživatele.
          </Alert>
          <Stepper index={activeStep}>
            {steps.map((step, index) => (
              <Step key={index}>
                <StepIndicator>
                  <StepStatus
                    complete={<StepIcon />}
                    incomplete={<StepNumber />}
                    active={<StepNumber />}
                  />
                </StepIndicator>
                <Box flexShrink="0">
                  <StepTitle>{step.title}</StepTitle>
                </Box>
                <StepSeparator />
              </Step>
            ))}
          </Stepper>
          <Card>
            <CardBody sx={{ p: 8 }}>
              <Stack spacing="6" as="form" onSubmit={handleSubmit(onSubmit)}>
                <Stack spacing="5">{steps[activeStep - 1].content}</Stack>
              </Stack>
            </CardBody>
          </Card>
        </Stack>
      </Container>
    </Box>
  );
};

export default SignUp;
